@@ -1,7 +1,7 @@
 /* GENIUS Calculator
- * Copyright (C) 1997-2002 George Lebl
+ * Copyright (C) 1997-2009 Jiri (George) Lebl
  *
- * Author: George Lebl
+ * Author: Jiri (George) Lebl
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 
 #include "parseutil.h"
 
-extern GSList *evalstack;
+extern GSList *gel_parsestack;
 
 
 gboolean
@@ -45,20 +45,20 @@ gp_push_func (gboolean vararg)
 	GSList * list = NULL;
 	int i = 0;
 	
-	val = stack_pop(&evalstack);
+	val = gel_stack_pop(&gel_parsestack);
 	if(!val)
 		return FALSE;
 
 
 	for(;;) {
-		tree = stack_pop(&evalstack);
-		if(tree && tree->type==EXPRLIST_START_NODE) {
+		tree = gel_stack_pop(&gel_parsestack);
+		if(tree && tree->type==GEL_EXPRLIST_START_NODE) {
 			gel_freetree(tree);
 			break;
 		}
 		/*we have gone all the way to the top and haven't found a
 		  marker or tree is not an ident node*/
-		if(!tree || tree->type != IDENTIFIER_NODE) {
+		if(!tree || tree->type != GEL_IDENTIFIER_NODE) {
 			if(tree) gel_freetree(tree);
 			g_slist_foreach(list,(GFunc)gel_freetree,NULL);
 			g_slist_free(list); 
@@ -69,14 +69,14 @@ gp_push_func (gboolean vararg)
 		i++;
 	}
 	
-	GET_NEW_NODE(tree);
+	GEL_GET_NEW_NODE(tree);
 
-	tree->type = FUNCTION_NODE;
+	tree->type = GEL_FUNCTION_NODE;
 	tree->func.func = d_makeufunc(NULL,val,list,i, NULL);
 	tree->func.func->context = -1;
 	tree->func.func->vararg = vararg;
 
-	stack_push(&evalstack,tree);
+	gel_stack_push(&gel_parsestack,tree);
 
 	return TRUE;
 }
@@ -91,19 +91,19 @@ gp_prepare_push_param (gboolean setfunc)
 	/* FIXME: setfunc not yet implemented */
 	g_assert ( ! setfunc);
 
-	val = stack_pop (&evalstack);
+	val = gel_stack_pop (&gel_parsestack);
 	if (val == NULL)
 		return FALSE;
 
-	ident = stack_pop (&evalstack);
+	ident = gel_stack_pop (&gel_parsestack);
 	if (ident == NULL)
 		return FALSE;
 
 	func = gel_makenum_null ();
 
-	stack_push (&evalstack, func);
-	stack_push (&evalstack, ident);
-	stack_push (&evalstack, val);
+	gel_stack_push (&gel_parsestack, func);
+	gel_stack_push (&gel_parsestack, ident);
+	gel_stack_push (&gel_parsestack, val);
 
 	return TRUE;
 }
@@ -114,25 +114,25 @@ gp_prepare_push_region_sep (void)
 {
 	GelETree *e1, *e2;
 
-	e2 = stack_pop (&evalstack);
-	e1 = stack_pop (&evalstack);
+	e2 = gel_stack_pop (&gel_parsestack);
+	e1 = gel_stack_pop (&gel_parsestack);
 
-	if (e2->type == OPERATOR_NODE &&
-	    e2->op.oper == E_REGION_SEP) {
+	if (e2->type == GEL_OPERATOR_NODE &&
+	    e2->op.oper == GEL_E_REGION_SEP) {
 		GelETree *a1 = e2->op.args;
 		GelETree *a2 = e2->op.args->any.next;
 		a1->any.next = NULL;
 		a2->any.next = NULL;
 		e2->op.args = NULL;
 		gel_freetree (e2);
-		stack_push (&evalstack, e1);
-		stack_push (&evalstack, a1);
-		stack_push (&evalstack, a2);
+		gel_stack_push (&gel_parsestack, e1);
+		gel_stack_push (&gel_parsestack, a1);
+		gel_stack_push (&gel_parsestack, a2);
 
 		return TRUE;
 	} else {
-		stack_push (&evalstack, e1);
-		stack_push (&evalstack, e2);
+		gel_stack_push (&gel_parsestack, e1);
+		gel_stack_push (&gel_parsestack, e2);
 
 		return FALSE;
 	}
@@ -140,19 +140,19 @@ gp_prepare_push_region_sep (void)
 
 /*pops the last expression, pushes a marker
   entry and puts the last expression back*/
-int
+gboolean
 gp_push_marker(GelETreeType markertype)
 {
-	GelETree * last_expr = stack_pop(&evalstack);
+	GelETree * last_expr = gel_stack_pop(&gel_parsestack);
 	GelETree * tree;
 	
 	if(!last_expr)
 		return FALSE;
        
-	GET_NEW_NODE(tree);
+	GEL_GET_NEW_NODE(tree);
 	tree->type = markertype;
-	stack_push(&evalstack,tree);
-	stack_push(&evalstack,last_expr);
+	gel_stack_push(&gel_parsestack,tree);
+	gel_stack_push(&gel_parsestack,last_expr);
 	return TRUE;
 }
 
@@ -161,42 +161,42 @@ void
 gp_push_marker_simple(GelETreeType markertype)
 {
 	GelETree *tree;
-	GET_NEW_NODE(tree);
+	GEL_GET_NEW_NODE(tree);
 	tree->type = markertype;
-	stack_push(&evalstack,tree);
+	gel_stack_push(&gel_parsestack,tree);
 }
 
 /*puts a spacer into the tree, spacers are just useless nodes to be removed
   before evaluation, they just signify where there were parenthesis*/
-int
+gboolean
 gp_push_spacer(void)
 {
-	GelETree * last_expr = stack_pop(&evalstack);
+	GelETree * last_expr = gel_stack_pop(&gel_parsestack);
 	
 	if(!last_expr)
 		return FALSE;
-	else if(last_expr->type == SPACER_NODE)
-		stack_push(&evalstack,last_expr);
+	else if(last_expr->type == GEL_SPACER_NODE)
+		gel_stack_push(&gel_parsestack,last_expr);
 	else {
 		GelETree * tree;
-		GET_NEW_NODE(tree);
-		tree->type = SPACER_NODE;
+		GEL_GET_NEW_NODE(tree);
+		tree->type = GEL_SPACER_NODE;
 		tree->sp.arg = last_expr;
-		stack_push(&evalstack,tree);
+		gel_stack_push(&gel_parsestack,tree);
 	}
 	return TRUE;
 }
 	
 /*gather all expressions up until a row start marker and push the
-  result as a MATRIX_ROW_NODE*/
-int
+  result as a GEL_MATRIX_ROW_NODE*/
+gboolean
 gp_push_matrix_row(void)
 {
 	GelETree *tree;
 	GelETree *row = NULL;
 	int i=0;
 	for(;;) {
-		tree = stack_pop(&evalstack);
+		tree = gel_stack_pop(&gel_parsestack);
 		/*we have gone all the way to the top and haven't found a
 		  marker*/
 		if(!tree) {
@@ -207,7 +207,7 @@ gp_push_matrix_row(void)
 			}
 			return FALSE;
 		}
-		if(tree->type==EXPRLIST_START_NODE) {
+		if(tree->type==GEL_EXPRLIST_START_NODE) {
 			gel_freetree(tree);
 			break;
 		}
@@ -215,20 +215,20 @@ gp_push_matrix_row(void)
 		row = tree;
 		i++;
 	}
-	GET_NEW_NODE(tree);
-	tree->type = MATRIX_ROW_NODE;
+	GEL_GET_NEW_NODE(tree);
+	tree->type = GEL_MATRIX_ROW_NODE;
 	tree->row.args = row;
 	tree->row.nargs = i;
 
-	stack_push(&evalstack,tree);
+	gel_stack_push(&gel_parsestack,tree);
 	
 	return TRUE;
 }
 	
 /*gather all expressions up until a row start marker and push the
   result as a matrix*/
-int
-gp_push_matrix(int quoted)
+gboolean
+gp_push_matrix(gboolean quoted)
 {
 	GelETree *tree;
 	int i,j;
@@ -242,7 +242,7 @@ gp_push_matrix(int quoted)
 	rows=0;
 	cols=0;
 	for(;;) {
-		tree = stack_pop(&evalstack);
+		tree = gel_stack_pop(&gel_parsestack);
 		/*we have gone all the way to the top and haven't found a
 		  marker*/
 		if(!tree) {
@@ -258,10 +258,10 @@ gp_push_matrix(int quoted)
 			g_slist_free(rowl);
 			/**/g_warning("BAD MATRIX, NO START MARKER");
 			return FALSE;
-		} else if(tree->type==MATRIX_START_NODE) {
+		} else if(tree->type==GEL_MATRIX_START_NODE) {
 			gel_freetree(tree);
 			break;
-		} else if(tree->type==MATRIX_ROW_NODE) {
+		} else if(tree->type==GEL_MATRIX_ROW_NODE) {
 			if(tree->row.nargs>cols)
 				cols = tree->row.nargs;
 			rowl = g_slist_prepend(rowl,tree->row.args);
@@ -294,15 +294,23 @@ gp_push_matrix(int quoted)
 		for(i=0,lix=liy->data;lix;i++,lix=lix->any.next) {
 			gel_matrix_index(matrix,i,j) = lix;
 		}
+		/* On non-quoted matrices fill the matrix with nulls
+		 * since those may be converted to zeros */
+		if ( ! quoted) {
+			for(;i < cols;i++) {
+				gel_matrix_index (matrix, i, j) =
+					gel_makenum_null ();
+			}
+		}
 	}
 	g_slist_free(rowl);
 	
-	GET_NEW_NODE(tree);
-	tree->type = MATRIX_NODE;
+	GEL_GET_NEW_NODE(tree);
+	tree->type = GEL_MATRIX_NODE;
 	tree->mat.matrix = gel_matrixw_new_with_matrix(matrix);
-	tree->mat.quoted = quoted?1:0;
+	tree->mat.quoted = quoted ? 1 : 0;
 	
-	stack_push(&evalstack,tree);
+	gel_stack_push(&gel_parsestack,tree);
 	return TRUE;
 }
 
@@ -312,8 +320,36 @@ void
 gp_push_null(void)
 {
 	GelETree *tree;
-	GET_NEW_NODE(tree);
-	tree->type = NULL_NODE;
+	GEL_GET_NEW_NODE(tree);
+	tree->type = GEL_NULL_NODE;
 
-	stack_push(&evalstack,tree);
+	gel_stack_push(&gel_parsestack,tree);
+}
+
+void
+gp_convert_identifier_to_bool (void)
+{
+	GelETree *val;
+
+	val = gel_stack_peek (&gel_parsestack);
+	if (val == NULL ||
+	    val->type != GEL_IDENTIFIER_NODE) {
+		/**/g_warning ("NO IDENTIFIER TO CONVERT TO TRY TO CONVERT BOOL");
+		return;
+	}
+	if (val->id.id == NULL ||
+	    val->id.id->token == NULL)
+		return;
+
+	if (strcmp (val->id.id->token, "true") == 0 ||
+	     strcmp (val->id.id->token, "True") == 0 ||
+	     strcmp (val->id.id->token, "TRUE") == 0) {
+		gel_emptytree (val);
+		gel_makenum_bool_from (val, TRUE);
+	} else if (strcmp (val->id.id->token, "false") == 0 ||
+		   strcmp (val->id.id->token, "False") == 0 ||
+		   strcmp (val->id.id->token, "FALSE") == 0) {
+		gel_emptytree (val);
+		gel_makenum_bool_from (val, FALSE);
+	}
 }
